/*
 * Copyright (c) 2005-2008 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.gen.ui.wizards.export;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.acceleo.gen.ui.AcceleoGenUIMessages;
import org.acceleo.gen.ui.AcceleoGenUiPlugin;
import org.acceleo.gen.ui.wizards.export.operation.AcceleoExportToStandaloneOperation;
import org.acceleo.gen.ui.wizards.export.page.AcceleoSelectProjectFileWizardPage;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;

import fr.obeo.acceleo.gen.template.TemplateConstants;
import fr.obeo.acceleo.gen.ui.wizards.AcceleoNewProjectWizardPage;
import fr.obeo.acceleo.gen.ui.wizards.AcceleoSelectFilesWizardPage;
import fr.obeo.acceleo.tools.resources.Resources;
import fr.obeo.acceleo.tools.strings.Int2;
import fr.obeo.acceleo.tools.strings.TextSearch;

/**
 * Wizard to export a generator project to a standalone folder.
 * 
 * @author www.obeo.fr
 * 
 */
public class AcceleoExportToStandaloneWizard extends Wizard implements IExportWizard {

	/**
	 * The current selection.
	 */
	protected ISelection selection;

	/**
	 * Project to export.
	 */
	private IProject selectedProject;

	/**
	 * The page to configure the destination project.
	 */
	protected AcceleoNewProjectWizardPage projectPage;

	/**
	 * The page to select mt files to export.
	 */
	protected AcceleoSelectProjectFileWizardPage selectMtFilesPage;

	/**
	 * The page to select resource file to export.
	 */
	protected AcceleoSelectFilesWizardPage selectFilesPage;

	/**
	 * Create the wizard.
	 */
	public AcceleoExportToStandaloneWizard() {
	}

	/* (non-Javadoc) */
	public boolean performFinish() {
		try {
			// Check if the user select mt files with no tag file in script
			// descriptor
			List selectedFiles = selectMtFilesPage.getSelectedFile();
			List filesWithNoFileTag = new ArrayList();
			for (Iterator it = selectedFiles.iterator(); it.hasNext();) {
				IFile currentFile = (IFile) it.next();
				Int2 position = TextSearch.getRegexSearch().indexOf(Resources.getFileContent(currentFile).toString(),
						TemplateConstants.SCRIPT_FILE + "[ \t\r\n]*" + TemplateConstants.SCRIPT_PROPERTY_ASSIGN + "[ \t\r\n]*" + TemplateConstants.LITERAL[0]);
				if (position.b() == -1) {
					filesWithNoFileTag.add(currentFile);
				}
			}
			if (filesWithNoFileTag.size() > 0) {
				String files = "";
				for (int i = 0; i < filesWithNoFileTag.size(); ++i) {
					files += ((IFile) filesWithNoFileTag.get(i)).getName();
					if (i < filesWithNoFileTag.size() - 1) {
						files += ", ";
					}
				}
				MessageDialog.openWarning(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), AcceleoGenUIMessages.getString("AcceleoExportToStandaloneWizard.NoFileDescriptorFound"),
						AcceleoGenUIMessages.getString("AcceleoExportToStandaloneWizard.NoFileTagDescriptor", new Object[] { files }));
			}
			AcceleoExportToStandaloneOperation exportOperation = new AcceleoExportToStandaloneOperation(selectedProject, selectMtFilesPage.getSelectedFile(), projectPage.getProjectName(),
					selectFilesPage.getSelection());
			PlatformUI.getWorkbench().getProgressService().run(false, false, exportOperation);
		} catch (Exception e) {
			AcceleoGenUiPlugin.getDefault().log(e, true);
		}

		return true;
	}

	/* non-javadoc */
	public void addPages() {
		addPage(projectPage);
		addPage(selectMtFilesPage);
		addPage(selectFilesPage);
	}

	/* non-javadoc */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
		if (selection instanceof StructuredSelection) {
			Object selectedObject = ((StructuredSelection) selection).getFirstElement();
			if (selectedObject instanceof IProject) {
				selectedProject = (IProject) selectedObject;
			} else if (selectedObject instanceof IJavaProject) {
				selectedProject = ((IJavaProject) selectedObject).getProject();
			}
		}
		setWindowTitle(AcceleoGenUIMessages.getString("AcceleoExportToStandaloneWizard.Title")); //$NON-NLS-1$
		projectPage = new AcceleoNewProjectWizardPage(AcceleoGenUIMessages.getString("AcceleoExportToStandaloneWizard.ProjectExportPage"), selectedProject.getName() + ".export");
		selectMtFilesPage = new AcceleoSelectProjectFileWizardPage(AcceleoGenUIMessages.getString("AcceleoExportToStandaloneWizard.SelectGen"), selectedProject);
		selectFilesPage = new AcceleoSelectFilesWizardPage(AcceleoGenUIMessages.getString("AcceleoExportToStandaloneWizard.SelectOtherFiles"), -1, -1, new String[] { "*" }, false);
	}

}
