/*
 * Copyright (c) 2008 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.gen.ui.wizards.export.page;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.acceleo.gen.ui.AcceleoGenUIMessages;
import org.acceleo.gen.ui.AcceleoGenUiPlugin;
import org.acceleo.gen.ui.wizards.export.util.AcceleoExportUtil;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTreeViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.model.WorkbenchContentProvider;

import fr.obeo.acceleo.gen.template.TemplateConstants;
import fr.obeo.acceleo.tools.resources.Resources;
import fr.obeo.acceleo.tools.strings.Int2;
import fr.obeo.acceleo.tools.strings.TextSearch;
import fr.obeo.acceleo.tools.ui.resources.AcceleoWorkbenchLabelProvider;
import fr.obeo.acceleo.tools.ui.resources.CheckboxTreeAndListGroup;

/**
 * The role of this wizard page is to make a collection of all of the checked
 * resources of the project.
 * 
 * @author www.obeo.fr
 */
public class AcceleoSelectProjectFileWizardPage extends WizardPage {

	/**
	 * The selection group.
	 */
	private CheckboxTreeAndListGroup checkboxGroup;

	/**
	 * The project.
	 */
	private IProject project;

	// Constants
	private final static int SELECTION_WIDGET_WIDTH = 400;

	private final static int SELECTION_WIDGET_HEIGHT = 300;

	/**
	 * Constructor.
	 * 
	 * @param pageName
	 *            is the page name
	 * @param project
	 *            is the project
	 */
	public AcceleoSelectProjectFileWizardPage(String pageName, IProject project) {
		super(pageName);
		setTitle(pageName);
		this.project = project;
	}

	/* (non-Javadoc) */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout containerLayout = new GridLayout();
		containerLayout.numColumns = 1;
		containerLayout.marginTop = 14;
		containerLayout.verticalSpacing = 9;
		containerLayout.marginLeft = 7;
		containerLayout.marginRight = 7;
		container.setLayout(containerLayout);
		AcceleoWorkbenchLabelProvider treeLabelProvider = new AcceleoWorkbenchLabelProvider();
		AcceleoWorkbenchLabelProvider listLabelProvider = new AcceleoWorkbenchLabelProvider();
		checkboxGroup = new CheckboxTreeAndListGroup(container, project, getContentProvider(IResource.FOLDER | IResource.PROJECT | IResource.ROOT), treeLabelProvider,
				getContentProvider(IResource.FILE), listLabelProvider, SWT.NONE, SELECTION_WIDGET_WIDTH, SELECTION_WIDGET_HEIGHT);
		checkboxGroup.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				dialogChanged();
			}
		});
		checkboxGroup.getTreeViewer().addOpenListener(new IOpenListener() {
			public void open(OpenEvent event) {
				ISelection selection = event.getSelection();
				if (!selection.isEmpty() && selection instanceof StructuredSelection) {
					checkboxGroup.getTreeViewer().expandToLevel(((StructuredSelection) selection).getFirstElement(), CheckboxTreeViewer.ALL_LEVELS);
				}
			}
		});

		container.addControlListener(new ControlListener() {
			public void controlMoved(ControlEvent e) {
			}

			public void controlResized(ControlEvent e) {
				TableColumn[] columns = checkboxGroup.getListTable().getColumns();
				for (int i = 0; i < columns.length; i++) {
					columns[i].pack();
				}
			}
		});

		dialogChanged();
		setControl(container);

		initSelection();

	}

	/**
	 * Returns the content provider.
	 * 
	 * @param resourceType
	 *            is the type of the resource
	 * @return the content provider
	 */
	private ITreeContentProvider getContentProvider(final int resourceType) {
		return new WorkbenchContentProvider() {
			public Object[] getChildren(Object o) {
				if (resourceType == IResource.FILE && o instanceof IPath) {
					return new Object[0];
				} else if (o instanceof IContainer) {
					IResource[] members = null;
					try {
						members = ((IContainer) o).members();
					} catch (CoreException e) {
						return new Object[0];
					}
					ArrayList results = new ArrayList();
					for (int i = 0; i < members.length; i++) {
						if ((members[i].getType() & resourceType) > 0 && isSignificant(members[i])) {
							if (members[i] instanceof IFile) {
								if (isValidFile((IFile) members[i])) {
									results.add(members[i]);
								}
							} else if (members[i] instanceof IContainer && countMembers((IContainer) members[i]) > 0) {
								results.add(members[i]);
							}
						}
					}
					return results.toArray();
				} else if (o instanceof List) {
					List result = new ArrayList();
					Iterator it = ((List) o).iterator();
					while (it.hasNext()) {
						Object element = it.next();
						if (element instanceof IContainer) {
							if (((IContainer) element).isAccessible() && countMembers((IContainer) element) > 0) {
								result.add(element);
							}
						} else {
							result.add(element);
						}
					}
					return result.toArray();
				} else {
					return new Object[0];
				}
			}
		};
	}

	private boolean isValidFile(IFile file) {
		boolean ret = false;
		// Check if the extension is ok
		ret = "mt".equals(file.getFileExtension());
		return ret;
	}

	private int countMembers(IContainer container) {
		try {
			IResource[] members = container.members();
			int result = 0;
			for (int i = 0; i < members.length; i++) {
				if (isSignificant(members[i])) {
					if (members[i] instanceof IFile) {
						if (isValidFile((IFile) members[i])) {
							result++;
						}
					} else if (members[i] instanceof IContainer) {
						result += countMembers((IContainer) members[i]);
					}
				}
			}
			return result;
		} catch (CoreException e) {
			return 0;
		}
	}

	private boolean isSignificant(IResource resource) {
		// We don't want output folder et folder wich name start with .
		return !(resource instanceof IFolder && (resource.equals(Resources.getOutputFolder(resource.getProject())) || resource.getName().startsWith("."))); //$NON-NLS-1$
	}

	/**
	 * Validates the changes on the page.
	 */
	private void dialogChanged() {
		List items = getAllCheckedListItems();
		if (items.size() <= 0) {
			updateStatus(AcceleoGenUIMessages.getString("AcceleoSelectProjectFileWizardPage.NotEnoughFiles")); //$NON-NLS-1$
			return;
		}
		updateStatus(null);
	}

	/**
	 * Updates the status of the page.
	 * 
	 * @param message
	 *            is the error message.
	 */
	private void updateStatus(String message) {
		setMessage(message);
		setPageComplete(message == null);
	}

	/**
	 * Gets all the checked items.
	 * 
	 * @return the checked items
	 */
	private List getAllCheckedListItems() {
		List items = new ArrayList();
		Iterator resultEnum = checkboxGroup.getAllCheckedListItems();
		while (resultEnum.hasNext()) {
			Object member = resultEnum.next();
			if (member instanceof IFile) {
				items.add(((IFile) member).getFullPath());
			}
		}
		return items;
	}

	/**
	 * Initialize the selection of the file to export
	 */
	private void initSelection() {
		// Init the selection of template to export
		Map selectionMap = new HashMap();
		try {
			List mtFiles = AcceleoExportUtil.listTemplateFiles(project, Resources.getOutputFolder(project));
			for (Iterator itFile = mtFiles.iterator(); itFile.hasNext();) {
				IFile currentFile = (IFile) itFile.next();
				// Check if the mt file contains the tag file
				Int2 position = TextSearch.getRegexSearch().indexOf(Resources.getFileContent(currentFile).toString(),
						TemplateConstants.SCRIPT_FILE + "[ \t\r\n]*" + TemplateConstants.SCRIPT_PROPERTY_ASSIGN + "[ \t\r\n]*" + TemplateConstants.LITERAL[0]);
				if (position.b() != -1 && position.e() != -1) {
					if (selectionMap.get(currentFile.getParent()) == null) {
						selectionMap.put(currentFile.getParent(), new ArrayList());
					}
					((List) (selectionMap.get(currentFile.getParent()))).add(currentFile);
				}
			}

		} catch (CoreException e) {
			AcceleoGenUiPlugin.getDefault().log(e, true);
		}
		checkboxGroup.updateSelections(selectionMap);
		checkboxGroup.getTreeViewer().expandAll();
		dialogChanged();
	}

	/**
	 * Get the selected files.
	 * 
	 * @return the selected files
	 */
	public List getSelectedFile() {
		List ret = new ArrayList();
		for (Iterator it = checkboxGroup.getAllCheckedListItems(); it.hasNext();) {
			Object currentSelection = (Object) it.next();
			if (currentSelection instanceof IFile) {
				ret.add(currentSelection);
			}
		}
		return ret;
	}

}
