/*
 * Copyright (c) 2005-2008 Obeo
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.gen.ui.wizards.export.template;

/**
 * Java JET generation argument.
 * 
 * @author www.obeo.fr
 * 
 */
public class AcceleoExportToStandaloneTemplateModel {

	/**
	 * Path of the EMT file.
	 */
	private String emtFile;

	/**
	 * Name of the EMT file. It is used to generate the name of the class.
	 */
	private String emtName;

	/**
	 * Package name of the generated class.
	 */
	private String packageName;

	/**
	 * Main package name of the meta model.
	 */
	private String modelPackageName;

	/**
	 * Indicates if the meta model is UML.
	 */
	private boolean umlModel;

	/**
	 * Absolute system path of the org.eclipse.uml2.uml.jar file.
	 */
	private String umlJarPath;

	/**
	 * Absolute system path of the org.eclipse.uml2.uml.resources.jar file.
	 */
	private String umlResourcesJarPath;

	/**
	 * Absolute system path of the org.eclipse.uml2.uml.common.jar file.
	 */
	private String umlCommonJarPath;

	/**
	 * Indicate if the meta model is ECORE.
	 */
	private boolean ecoreModel;

	/**
	 * Ecore file path.
	 */
	private String ecorePath;

	/**
	 * Gets the EMT file.
	 * 
	 * @return the EMT file
	 */
	public String getEmtFile() {
		return emtFile;
	}

	/**
	 * Sets the EMT file.
	 * 
	 * @param emtFile
	 *            is the EMT file
	 */
	public void setEmtFile(String emtFile) {
		this.emtFile = emtFile;
	}

	/**
	 * Gets the EMT file name. It is used to generate the name of the class.
	 * 
	 * @return the EMT file name
	 */
	public String getEmtName() {
		return emtName;
	}

	/**
	 * Sets the EMT file name
	 * 
	 * @param emtName
	 *            is EMT file name
	 */
	public void setEmtName(String emtName) {
		this.emtName = emtName;
	}

	/**
	 * Gets the package name of the generated class.
	 * 
	 * @return the package name
	 */
	public String getPackageName() {
		return packageName;
	}

	/**
	 * Sets the package name of the generated class
	 * 
	 * @param packageName
	 *            is the package name
	 */
	public void setPackageName(String packageName) {
		this.packageName = packageName;
	}

	/**
	 * Gets the main package name of the meta model.
	 * 
	 * @return the main package name of the meta model
	 */
	public String getModelPackageName() {
		return modelPackageName;
	}

	/**
	 * Sets the main package name of the meta model
	 * 
	 * @param modelPackageName
	 *            is the main package name of the meta model
	 */
	public void setModelPackageName(String modelPackageName) {
		this.modelPackageName = modelPackageName;
	}

	/**
	 * Indicates if the meta model is UML.
	 * 
	 * @return true if the meta model is UML.
	 */
	public boolean isUmlModel() {
		return umlModel;
	}

	/**
	 * Are We working with the UML meta model?
	 * 
	 * @param umlModel
	 *            is true if we are working with the UML meta model
	 */
	public void setUmlModel(boolean umlModel) {
		this.umlModel = umlModel;
	}

	/**
	 * Gets the absolute system path of the org.eclipse.uml2.uml.jar file.
	 * 
	 * @return the absolute path
	 */
	public String getUmlJarPath() {
		return umlJarPath;
	}

	/**
	 * Sets the absolute system path of the org.eclipse.uml2.uml.jar file.
	 * 
	 * @param umlJarPath
	 *            is the absolute path
	 */
	public void setUmlJarPath(String umlJarPath) {
		this.umlJarPath = umlJarPath;
	}

	/**
	 * Gets the absolute system path of the org.eclipse.uml2.uml.resources.jar
	 * file.
	 * 
	 * @return the absolute path
	 */
	public String getUmlResourcesJarPath() {
		return umlResourcesJarPath;
	}

	/**
	 * Sets the absolute system path of the org.eclipse.uml2.uml.resources.jar
	 * file.
	 * 
	 * @param umlResourcesJarPath
	 *            is the absolute path
	 */
	public void setUmlResourcesJarPath(String umlResourcesJarPath) {
		this.umlResourcesJarPath = umlResourcesJarPath;
	}

	/**
	 * Gets the absolute system path of the org.eclipse.uml2.uml.common.jar
	 * file.
	 * 
	 * @return the absolute path
	 */
	public String getUmlCommonJarPath() {
		return umlCommonJarPath;
	}

	/**
	 * Sets the absolute system path of the org.eclipse.uml2.uml.common.jar
	 * file.
	 * 
	 * @param umlCommonJarPath
	 *            is the absolute path
	 */
	public void setUmlCommonJarPath(String umlCommonJarPath) {
		this.umlCommonJarPath = umlCommonJarPath;
	}

	/**
	 * Indicates if the meta model is ECORE.
	 * 
	 * @return true if the meta model is ECORE.
	 */
	public boolean isEcoreModel() {
		return ecoreModel;
	}

	/**
	 * Are We working with the ECORE meta model?
	 * 
	 * @param ecoreModel
	 *            is true if we are working with the ECORE meta model
	 */
	public void setEcoreModel(boolean ecoreModel) {
		this.ecoreModel = ecoreModel;
	}

	/**
	 * Gets the ecore file path.
	 * 
	 * @return the ecore file path
	 */
	public String getEcorePath() {
		return ecorePath;
	}

	/**
	 * Sets the ecore file path.
	 * 
	 * @param ecorePath
	 *            is the ecore file path
	 */
	public void setEcorePath(String ecorePath) {
		this.ecorePath = ecorePath;
	}
}
