/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.engine.eval.imports.services;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

import java.util.HashMap;
import java.util.Map;
import java.util.Stack;


/**
 * System services for ENode elements.
 * 
 * @author www.obeo.fr
 * 
 */
public class ContextServices {

	/** Context stack. */
	private Stack<ENode> stack = new Stack<ENode>();

	/**
	 * Context map.
	 */
	private Map<String, ENode> context = new HashMap<String, ENode>();

	/**
	 * Constructor.
	 */
	public ContextServices() {
	}

	/**
	 * Puts the current node in the context for the given key, and returns an
	 * empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 * @deprecated
	 */
	public void nPut(ENode node, String key) {
		put(node, key);
	}

	/**
	 * Puts the current node in the context for the given key, and returns an
	 * empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 */
	public void put(ENode node, String key) {
		context.put(key, node);
	}

	/**
	 * Gets the node for the given key in the context.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 * @return the node of the context
	 * @throws ENodeCastException
	 *             if cast error occurs
	 * @deprecated
	 */
	public ENode nGet(ENode node, ENode key) throws ENodeCastException {
		ENode associatedNode;
		if (key.isInt()) {
			if (node.isList()) {
				ENodeList list = node.getList();
				ENode result = list.get(key.getInt());
				if (result != null) {
					associatedNode = result;
				} else {
					associatedNode = new ENode(ENode.EMPTY, node);
				}
			} else if (key.getInt() == 0) {
				associatedNode = node;
			} else {
				associatedNode = new ENode(ENode.EMPTY, node);
			}
		} else if (key.isString()) {
			// deprecated nGet(ENode node, String key)
			associatedNode = get(node, key.getString());
		} else {
			// deprecated nGet(ENode node, String key)
			associatedNode = get(node, (String) key.getAdapterValue(String.class));
		}
		return associatedNode;
	}

	/**
	 * Gets the node for the given key in the context.
	 * 
	 * @param node
	 *            is the current node
	 * @param key
	 *            is the key in the context
	 * @return the node of the context
	 */
	public ENode get(ENode node, String key) {
		ENode result = context.get(key);
		if (result != null) {
			return result.copy();
		} else {
			return new ENode(ENode.EMPTY, node);
		}
	}

	/**
	 * Pushes the current node into the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @deprecated
	 */
	public void nPush(ENode node) {
		push(node);
	}

	/**
	 * Pushes the current node into the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 */
	public void push(ENode node) {
		stack.push(node);
	}

	/**
	 * Pops the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 * @deprecated
	 */
	public void nPop(ENode node) {
		pop(node);
	}

	/**
	 * Pops the context, and returns an empty string.
	 * 
	 * @param node
	 *            is the current node
	 */
	public void pop(ENode node) {
		stack.pop();
	}

	/**
	 * Return the last node pushed into the context, and returns an empty
	 * string.
	 * 
	 * @param node
	 *            is the current node
	 * @return an empty string
	 * @deprecated
	 */
	public ENode nPeek(ENode node) {
		return peek(node);
	}

	/**
	 * Return the last node pushed into the context, and returns an empty
	 * string.
	 * 
	 * @param node
	 *            is the current node
	 * @return an empty string
	 */
	public ENode peek(ENode node) {
		if (!stack.isEmpty()) {
			return stack.peek().copy();
		} else {
			return new ENode("", node); //$NON-NLS-1$
		}
	}

}
