/*
 * Copyright (c) 2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */
package org.acceleo.engine.template.expressions.impl.spec;

import fr.obeo.acceleo.gen.template.eval.ENode;
import fr.obeo.acceleo.gen.template.eval.ENodeCastException;
import fr.obeo.acceleo.gen.template.eval.ENodeException;
import fr.obeo.acceleo.gen.template.eval.ENodeIterator;
import fr.obeo.acceleo.gen.template.eval.ENodeList;

import java.util.Iterator;

import org.acceleo.engine.AcceleoEngineMessages;
import org.acceleo.engine.context.EvalContext;
import org.acceleo.engine.context.ScriptContext;
import org.acceleo.engine.eval.imports.EvalJavaService;
import org.acceleo.engine.template.impl.spec.IEvaluable;
import org.acceleo.template.core.Script;
import org.acceleo.template.expressions.Expression;
import org.acceleo.template.expressions.impl.CallImpl;
import org.acceleo.tools.factory.EFactory;
import org.acceleo.tools.factory.FactoryException;
import org.acceleo.tools.strings.Int2;
import org.acceleo.tools.template.TemplateConstants;

/**
 * A representation of the model object '<em><b>Call</b></em>'.
 * 
 * @author <a href="mailto:yvan.lussaud@obeo.fr">Yvan Lussaud</a>
 * 
 */
public class TCall extends CallImpl implements IEvaluable {
	/**
	 * {@inheritDoc}
	 */
	public ENode gen(ENode root, EvalContext evalContext) throws ENodeException, FactoryException {
		ENode ret = null;

		if (name != null && !"".equals(name)) { //$NON-NLS-1$
			// Parameters management
			ENode[] args = new ENode[getArguments().size()];
			for (int j = 0; j < getArguments().size(); ++j) {
				args[j] = ((IEvaluable) getArguments().get(j)).gen((ENode) evalContext.getCurrentScriptContext().contextPeek(ScriptContext.CURRENT_NODE), evalContext);
			}

			// call to args
			if (TemplateConstants.LINK_NAME_ARGS.equals(name)) {
				ENode[] templateArgs = (ENode[]) evalContext.getCurrentScriptContext().contextPeek(ScriptContext.TEMPLATE_ARGS);
				if (templateArgs == null) {
					templateArgs = new ENode[] {};
				}
				try {
					int argIndex = args[0].getInt();
					if (argIndex < templateArgs.length) {
						ret = templateArgs[argIndex];
					}
				} catch (ENodeCastException e) {
					throw new ENodeException(AcceleoEngineMessages.getString("TCall.InvalidArgument"), new Int2(begin, end), getTemplate(), root, true); //$NON-NLS-1$
				}
				// call to i
			} else if (TemplateConstants.LINK_NAME_INDEX.equals(name)) {
				Integer index = (Integer) evalContext.getCurrentScriptContext().contextPeek(ScriptContext.WHILE_INDEX);
				if (index == null) {
					index = new Integer(0);
				}
				ret = new ENode(index.intValue(), root);
			} else if (TemplateConstants.USER_BEGIN_NAME.equals(name) && arguments.size() == 0) {
				ret = new ENode(TemplateConstants.DEFAULT_USER_BEGIN, root);
			} else if (TemplateConstants.USER_END_NAME.equals(name) && arguments.size() == 0) {
				ret = new ENode(TemplateConstants.DEFAULT_USER_END, root);
			} else {
				// Try to find a script from the EMT
				if (ret == null && ("".equals(prefix) || TemplateConstants.LINK_PREFIX_SCRIPT.equals(prefix))) { //$NON-NLS-1$
					ret = findScript(root, evalContext, args);
				}
				// Try to find a java service
				if (ret == null && ("".equals(prefix) || TemplateConstants.LINK_PREFIX_JAVA.equals(prefix))) { //$NON-NLS-1$
					ret = findService(root, evalContext, args, true);
					if (ret == null) {
						ret = findDefaultService(root, evalContext, args, true);
					}
				}
				// Try to find a metamodel element (attribute, reference, ...)
				if (ret == null && ("".equals(prefix) || TemplateConstants.LINK_PREFIX_METAMODEL.equals(prefix) || TemplateConstants.LINK_PREFIX_METAMODEL_SHORT.equals(prefix))) { //$NON-NLS-1$
					ret = findMetamodel(root, evalContext, args);
				}
			}
		} else { // an expression is stored in first argument TODO should
			// change that in EMT
			ret = ((IEvaluable) getArguments().get(0)).gen(root, evalContext);
		}

		if (ret == null) {
			throw new ENodeException(AcceleoEngineMessages.getString("TCall.UnresolvedCall", new Object[] { name, }), new Int2(getBegin(), getEnd()), getTemplate(), root, true); //$NON-NLS-1$
		}
		return applyFilter(ret, root, evalContext);
	}

	/**
	 * Try to find a metamodel element that correspond to this Call.
	 * 
	 * @param root
	 *            Current ENode
	 * @param evalContext
	 *            Current evaluation context
	 * @param args
	 *            Call arguments
	 * @return a new ENode with the interpreted value
	 * @throws FactoryException
	 *             if the given feature could not be accessed.
	 */
	private ENode findMetamodel(ENode root, EvalContext evalContext, ENode[] args) throws FactoryException {
		ENode ret = null;
		if (root.isList()) {
			ENodeList resList = new ENodeList();
			ENodeList list = null;
			try {
				list = root.getList();
			} catch (ENodeCastException e) {
				// impossible
			}
			for (int j = 0; j < list.size(); ++j) {
				ENode listCurrent = findFeature(list.get(j));
				if (listCurrent.getValue() != ENode.EMPTY) {
					resList.add(listCurrent);
				}
			}
			ret = new ENode(resList, root);
		} else {
			ret = findFeature(root);
		}
		return ret;
	}

	/**
	 * Try to find a service that correspond to this Call.
	 * 
	 * @param root
	 *            Current ENode
	 * @param evalContext
	 *            Current evaluation context
	 * @param args
	 *            Call arguments
	 * @return a new ENode with the interpreted value
	 * @throws ENodeException
	 *             if the service can't be interpreted.
	 * @throws FactoryException
	 *             if the service expression can't be interpreted.
	 */
	private ENode findService(ENode root, EvalContext evalContext, ENode[] args, boolean recursiveSearch) throws FactoryException, ENodeException {
		// TODO revoir le mécanisme de choix d'un
		// service, c.f. eGet de AbstractScript
		ENode evalResult = null;

		for (Iterator<EvalJavaService> itJavaService = evalContext.getCurrentServices().iterator(); itJavaService.hasNext() && evalResult == null;) {
			EvalJavaService javaService = itJavaService.next();
			if (recursiveSearch || !javaService.hasScriptContext()) {
				javaService.setMode(EvalJavaService.MODE_ENODE);
				evalResult = javaService.gen(this, root, evalContext, args);
				if (evalResult == null) {
					javaService.setMode(EvalJavaService.MODE_LIST);
					evalResult = javaService.gen(this, root, evalContext, args);
				}
			}
		}
		return evalResult;
	}

	private ENode findDefaultService(ENode root, EvalContext evalContext, ENode[] args, boolean recursiveSearch) throws FactoryException, ENodeException {
		ENode evalResult = null;

		for (Iterator<EvalJavaService> itJavaService = evalContext.getCurrentServices().iterator(); itJavaService.hasNext() && evalResult == null;) {
			EvalJavaService javaService = itJavaService.next();
			javaService.setMode(EvalJavaService.MODE_DEFAULT);
			if ((!recursiveSearch && !javaService.hasScriptContext()) || (recursiveSearch)) {
				if (!root.isList()) {
					evalResult = javaService.gen(this, root, evalContext, args);
				} else if (root.isList()) {
					ENodeIterator it = null;
					try {
						it = root.getList().iterator();
					} catch (ENodeCastException e) {
						// impossible
					}
					ENodeList resList = new ENodeList();
					while (it.hasNext()) {
						ENode child = it.next();
						ENode childResult = null;
						childResult = findDefaultService(child, evalContext, args, recursiveSearch);

						if (childResult != null) {
							resList.add(childResult);
						}
					}
					if (resList.size() > 0) {
						evalResult = new ENode(resList, root);
					}
				}
			}
		}
		return evalResult;
	}

	/**
	 * Try to find a Script that correspond to this Call.
	 * 
	 * @param root
	 *            Current ENode
	 * @param evalContext
	 *            Current evaluation context
	 * @param args
	 *            Call arguments
	 * @return a new ENode with the interpreted value
	 * @throws ENodeException
	 *             if the script expression can't be interpreted.
	 * @throws FactoryException
	 *             if the script expression can't be interpreted.
	 */
	private ENode findScript(ENode root, EvalContext evalContext, ENode[] args) throws ENodeException, FactoryException {
		ENode ret = null;
		boolean scriptFound = false;
		ENode res = null;
		if (root.isEObject()) {
			Script script;
			try {
				script = evalContext.getScript(root.getEObject(), name);
			} catch (ENodeCastException e) {
				script = null;
			}
			if (script != null) {
				evalContext.startScript(script);
				try {
					evalContext.getCurrentScriptContext().contextPush(ScriptContext.TEMPLATE_ARGS, args);
					res = ((IEvaluable) script).gen(root, evalContext);
				} finally {
					evalContext.endScript();
				}
				if (res != null) {
					scriptFound = true;
				}
			}
		} else if (root.isList()) {
			ENodeList resList = new ENodeList();
			ENode node = null;

			ENodeList list = null;
			try {
				list = root.getList();
			} catch (ENodeCastException e1) {
				// can't happen
			}

			for (int j = 0; j < list.size(); ++j) {
				node = list.get(j);
				if (node.isEObject()) {
					Script script = null;
					try {
						script = evalContext.getScript(node.getEObject(), name);
					} catch (ENodeCastException e) {
						// script = null;
					}
					if (script != null) {
						evalContext.startScript(script);
						ENode listCurrent = null;
						try {
							evalContext.getCurrentScriptContext().contextPush(ScriptContext.TEMPLATE_ARGS, args);
							listCurrent = ((IEvaluable) script).gen(node, evalContext);
						} finally {
							evalContext.endScript();
						}
						if (listCurrent != null) {
							scriptFound = true;
							resList.add(listCurrent);
						}
					}
				}
			}
			if (scriptFound) {
				res = new ENode(resList, root);
			}
		}
		if (scriptFound) {
			ret = res;
		}
		return ret;
	}

	/**
	 * Apply the filter to the ENode if any in this call.
	 * 
	 * @param nodeToFilter
	 *            ENode to filter
	 * @param root
	 *            Current ENode
	 * @param evalContext
	 *            Current evaluation context
	 * @return a new ENode corresponding to the filtered content
	 * @throws ENodeException
	 *             if the filter expression can't be interpreted.
	 * @throws FactoryException
	 *             if the filter expression can't be interpreted.
	 */
	private ENode applyFilter(ENode nodeToFilter, ENode root, EvalContext evalContext) throws ENodeException, FactoryException {
		ENode ret = nodeToFilter;
		try {
			if (filter != null) {
				if (!nodeToFilter.isList()) {
					ENode filterValue = ((IEvaluable) filter).gen(nodeToFilter, evalContext);
					Object boolValue = filterValue.getAdapterValue(boolean.class);
					if (!(boolValue instanceof Boolean && ((Boolean) boolValue).booleanValue())) {
						ret = new ENode(ENode.EMPTY, root);
					}
				} else { // is list
					ENodeList filteredList = new ENodeList();
					ENodeList list = null;
					try {
						list = nodeToFilter.getList();
					} catch (ENodeCastException e) {
						// can't happen
					}
					for (int j = 0; j < list.size(); ++j) {
						ENode listCurrent = list.get(j);
						ENode filterValue = ((IEvaluable) filter).gen(listCurrent, evalContext);
						Object boolValue = filterValue.getAdapterValue(boolean.class);
						if (boolValue instanceof Boolean && ((Boolean) boolValue).booleanValue()) {
							filteredList.add(listCurrent);
						}
					}
					ret = new ENode(filteredList, root);
				}
			}
		} catch (ENodeCastException e) {
			throw new ENodeException(AcceleoEngineMessages.getString("TCall.InvalidArgument"), new Int2(begin, end), getTemplate(), nodeToFilter, true); //$NON-NLS-1$
		}
		return ret;
	}

	/**
	 * Try to find a Feature with the same name of this Call.
	 * 
	 * @param root
	 *            the ENode containing the EObject that could contain the
	 * @return a new ENode with the interpreted value
	 * @throws FactoryException
	 *             if the given feature could not be accessed.
	 */
	// TODO reprendre le code du TemplateCallExpression
	private ENode findFeature(ENode root) {
		ENode ret = new ENode(ENode.EMPTY, root);
		Object retValue = null;
		if (!root.isNull()) {
			try {
				retValue = EFactory.eGet(root.getEObject(), name);
			} catch (ENodeCastException e) {
				// never catch
			} catch (FactoryException e) {
				// never catch
			}
			ret = ENode.createTry(retValue, root);
		}
		return ret;
	}

	/**
	 * {@inheritDoc}
	 */
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		if (prefix != null && prefix.length() > 0) {
			buffer.append(prefix);
			buffer.append(TemplateConstants.LINK_PREFIX_SEPARATOR);
		}
		buffer.append(name);
		if (arguments != null && arguments.size() > 0) {
			buffer.append(TemplateConstants.PARENTH[0]);
			Iterator<Expression> it = arguments.iterator();
			while (it.hasNext()) {
				Expression argument = it.next();
				buffer.append(argument.toString());
				if (it.hasNext()) {
					buffer.append(TemplateConstants.ARG_SEP);
				}
			}
			buffer.append(TemplateConstants.PARENTH[1]);
		}
		if (this.filter != null) {
			buffer.append(TemplateConstants.BRACKETS[0]);
			buffer.append(this.filter.toString());
			buffer.append(TemplateConstants.BRACKETS[1]);
		}
		return buffer.toString();
	}
}
