/*
 * Copyright (c) 2005-2008 Obeo.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Obeo - initial API and implementation
 */

package org.acceleo.gen.phantom;

import java.io.Externalizable;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * The .launcher phantom file.
 * 
 * @author www.obeo.fr
 * 
 */
public class ChainPhantom implements Externalizable {

	/** Serial used for unserialization checks. */
	private static final long serialVersionUID = 2L;

	/**
	 * The chain used to generate the files.
	 */
	protected Map<String, List<String>> chain2files = new HashMap<String, List<String>>();

	/**
	 * Clears all information about the chain.
	 * 
	 * @param chain
	 *            is the chain
	 */
	public void clear(File chain) {
		chain2files.remove(chain.getPath());
	}

	/**
	 * Adds a file generated by the given chain.
	 * 
	 * @param chain
	 *            is the chain
	 * @param file
	 *            is the generated file
	 */
	public void add(File chain, File file) {
		String key = chain.getPath();
		String value = file.getPath();
		List<String> files = chain2files.get(key);
		if (files == null) {
			files = new ArrayList<String>();
			chain2files.put(key, files);
		}
		if (!files.contains(value)) {
			files.add(value);
		}
	}

	/**
	 * Indicates if the given file has been generated.
	 * 
	 * @param file
	 *            is the file to test
	 * @return true if the given file has been generated
	 */
	public boolean isGenerated(File file) {
		String value = file.getPath();
		for (Entry<String, List<String>> entry : chain2files.entrySet()) {
			List<String> files = entry.getValue();
			if (files.contains(value)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	public void readExternal(ObjectInput arg) throws IOException, ClassNotFoundException {
		int sizeMap = arg.readInt();
		for (int i = 0; i < sizeMap; i++) {
			String key = arg.readUTF();
			int sizeList = arg.readInt();
			List<String> value = new ArrayList<String>(sizeList);
			for (int j = 0; j < sizeList; j++) {
				value.add(arg.readUTF());
			}
			chain2files.put(key, value);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void writeExternal(ObjectOutput arg) throws IOException {
		arg.writeInt(chain2files.size());
		for (Entry<String, List<String>> entry : chain2files.entrySet()) {
			String key = entry.getKey();
			List<String> files = entry.getValue();
			arg.writeUTF(key);
			arg.writeInt(files.size());
			for (String file : files) {
				arg.writeUTF(file);
			}
		}
	}

}
